/*
 * EditorWindow.java
 *
 * Created on July 14, 2006, 7:07 PM
 *
 * @author Paul Kratt
 * @desc The editor window. Controls the functionality of the rest of the program.
 *
 * Note: This code was released as open source on January 6th 2007.
 * There is no license behind it. The only restrictions I ask are that it may
 * not be sold and that some form of credit towards me remains if you use it.
 */

package trillianskinviewer;

import java.awt.Image;
import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;
import javax.swing.*;

public class EditorWindow extends javax.swing.JFrame {
    private ArrayList<String> setdata; //Settings file contents
    private File skinfolder; //Skin folder, if loaded
    private boolean loaded;
    private JPanel lastPreview = null;
    
    /** Creates new form EditorWindow */
    public EditorWindow() {
        initComponents();
        
        setdata = new ArrayList();
        
        loaded=false;
    }
    
    /** Grab the values for a specifc slicable object, and populate a SliceableItem
     *using those values.
     */
    public void populateSliceItem(String itemname,String colorname,Image itemImage){
        int templine;
        SliceableItem myitem = new SliceableItem();
        
        //Get image region
        templine = findLine(itemname+"-Region-top");
        myitem.setTop(getValue(templine));
        myitem.setLine_top(templine);
        templine = findLine(itemname+"-Region-bottom");
        myitem.setBottom(getValue(templine));
        myitem.setLine_bottom(templine);
        templine = findLine(itemname+"-Region-left");
        myitem.setLeft(getValue(templine));
        myitem.setLine_left(templine);
        templine = findLine(itemname+"-Region-right");
        myitem.setRight(getValue(templine));
        myitem.setLine_right(templine);
        
        //Get slice area
        templine = findLine(itemname+"-Slice-top");
        myitem.setSlicetop(getValue(templine));
        myitem.setLine_slicetop(templine);
        templine = findLine(itemname+"-Slice-bottom");
        myitem.setSlicebottom(getValue(templine));
        myitem.setLine_slicebottom(templine);
        templine = findLine(itemname+"-Slice-left");
        myitem.setSliceleft(getValue(templine));
        myitem.setLine_sliceleft(templine);
        templine = findLine(itemname+"-Slice-right");
        myitem.setSliceright(getValue(templine));
        myitem.setLine_sliceright(templine);
        
        //Get slice width
        templine = findLine(itemname+"-SliceWidth-top");
        myitem.setLine_widthtop(templine);
        templine = findLine(itemname+"-SliceWidth-bottom");
        myitem.setLine_widthbottom(templine);
        templine = findLine(itemname+"-SliceWidth-left");
        myitem.setLine_widthleft(templine);
        templine = findLine(itemname+"-SliceWidth-right");
        myitem.setLine_widthright(templine);
        
        //Get foreground colors
        templine = findLine(colorname+"-ForeColor-red");
        myitem.setForered(getValue(templine));
        myitem.setLine_forered(templine);
        templine = findLine(colorname+"-ForeColor-green");
        myitem.setForegreen(getValue(templine));
        myitem.setLine_foregreen(templine);
        templine = findLine(colorname+"-ForeColor-blue");
        myitem.setForeblue(getValue(templine));
        myitem.setLine_foreblue(templine);
        
        //Get background colors
        templine = findLine(colorname+"-BackColor-red");
        myitem.setBackred(getValue(templine));
        myitem.setLine_backred(templine);
        templine = findLine(colorname+"-BackColor-green");
        myitem.setBackgreen(getValue(templine));
        myitem.setLine_backgreen(templine);
        templine = findLine(colorname+"-BackColor-blue");
        myitem.setBackblue(getValue(templine));
        myitem.setLine_backblue(templine);
        
        //Fix bad boundary values now, otherwise preview will lock-up.
        myitem.boundsCheck();
        myitem.setSettingData(setdata);
        
        //Turn on the preview!
        myitem.setItemImage(itemImage);
        if(lastPreview != null)
            infPreview.remove(lastPreview);
        ItemPreview tempPreviewer = new ItemPreview(myitem);
        infPreview.add(tempPreviewer);
        myitem.setMyviewer(tempPreviewer);
        lastPreview = tempPreviewer;
        
        //Show the editor(s)
        tabEditors.add("Value editor",new SliceableEdit(myitem));
    }
    
    /** Find a setting line based on the property of it that we need.
     *@param property The name of the property to look for, of course!
     *@return The line number in the arraylist
     */
    public int findLine(String property){
        int retval = -1;
        for(int i=0; i<setdata.size();i++){
            if(setdata.get(i).indexOf("!ENTITY")>0 && setdata.get(i).indexOf(property)>0){
                retval=i; //Found it!
                break;
            }
        }
        return retval;
    }
    
    /** Get the value present on the specified line **/
    public int getValue(int line){
        int retval = 0;
        if(line>=0){
            String temp = setdata.get(line);

            temp = temp.substring(0,temp.lastIndexOf("'"));
            temp = temp.substring(temp.lastIndexOf("'")+1);
            try {
                //Now, we potentially have our value isolated into a string
                retval = Double.valueOf(temp).intValue();
            } catch (NumberFormatException ex) {
                ex.printStackTrace();
            } //We want to read it as a double in case the user used another
            //skinning program that I've seen spit out doubles for some reason.
            //The format documents on the Wiki claim integers, and that totally makes sense here.
        }
        else{
            retval=-1;
        }
        
        return retval;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        sptWindowPanes = new javax.swing.JSplitPane();
        pnlWinRight = new javax.swing.JPanel();
        pnlPreviewHolder = new javax.swing.JPanel();
        infPreview = new javax.swing.JInternalFrame();
        tabEditors = new javax.swing.JTabbedPane();
        scrSkinItems = new javax.swing.JScrollPane();
        lstSkinItems = new javax.swing.JList();
        mnuBar = new javax.swing.JMenuBar();
        mnuFile = new javax.swing.JMenu();
        mnuFileAbout = new javax.swing.JMenuItem();
        mnuFileSeparator = new javax.swing.JSeparator();
        mnuFileOpen = new javax.swing.JMenuItem();
        mnuFileSave = new javax.swing.JMenuItem();
        mnuFileSep2 = new javax.swing.JSeparator();
        mnuFileExit = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Trillian Skin Editor");
        sptWindowPanes.setDividerLocation(120);
        pnlPreviewHolder.setLayout(null);

        infPreview.getContentPane().setLayout(new java.awt.GridLayout(1, 0));

        infPreview.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        infPreview.setResizable(true);
        infPreview.setTitle("Preview");
        infPreview.setVisible(true);
        pnlPreviewHolder.add(infPreview);
        infPreview.setBounds(40, 10, 160, 100);

        org.jdesktop.layout.GroupLayout pnlWinRightLayout = new org.jdesktop.layout.GroupLayout(pnlWinRight);
        pnlWinRight.setLayout(pnlWinRightLayout);
        pnlWinRightLayout.setHorizontalGroup(
            pnlWinRightLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(pnlPreviewHolder, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 331, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, tabEditors, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 331, Short.MAX_VALUE)
        );
        pnlWinRightLayout.setVerticalGroup(
            pnlWinRightLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(pnlWinRightLayout.createSequentialGroup()
                .add(tabEditors, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 298, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(pnlPreviewHolder, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 154, Short.MAX_VALUE))
        );
        sptWindowPanes.setRightComponent(pnlWinRight);

        lstSkinItems.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Default Bevel", "Default Pressed", "Normal Button", "Over Button", "Pushed Button", "Disabled Button", "Display Box", "Text Box", "Frame", "Item List", "List", "Menubar", "Menu Item", "Normal Menu Button", "Over Menu Button", "Pushed Menu Button", "Window", "Window Interior" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        lstSkinItems.setEnabled(false);
        lstSkinItems.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                lstSkinItemsValueChanged(evt);
            }
        });

        scrSkinItems.setViewportView(lstSkinItems);

        sptWindowPanes.setLeftComponent(scrSkinItems);

        mnuFile.setText("File");
        mnuFileAbout.setText("About box...");
        mnuFileAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuFileAboutActionPerformed(evt);
            }
        });

        mnuFile.add(mnuFileAbout);

        mnuFile.add(mnuFileSeparator);

        mnuFileOpen.setText("Open Skin Folder...");
        mnuFileOpen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuFileOpenActionPerformed(evt);
            }
        });

        mnuFile.add(mnuFileOpen);

        mnuFileSave.setText("Save...");
        mnuFileSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuFileSaveActionPerformed(evt);
            }
        });

        mnuFile.add(mnuFileSave);

        mnuFile.add(mnuFileSep2);

        mnuFileExit.setText("Exit");
        mnuFileExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuFileExitActionPerformed(evt);
            }
        });

        mnuFile.add(mnuFileExit);

        mnuBar.add(mnuFile);

        setJMenuBar(mnuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(sptWindowPanes, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 457, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(sptWindowPanes, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 460, Short.MAX_VALUE)
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void mnuFileAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuFileAboutActionPerformed
        JOptionPane msgbox = new JOptionPane();
        
        msgbox.showMessageDialog(this,"Abandoned Trillian skin editor program\nVersion 0.1 alpha\n2006 Paul Kratt");
    }//GEN-LAST:event_mnuFileAboutActionPerformed

    private void mnuFileSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuFileSaveActionPerformed
        File setfile = new File(skinfolder.getPath()+skinfolder.separator+"settings.ini");
        
        try {
            PrintWriter writer = new PrintWriter(setfile);

            for(int i=0; i<setdata.size();i++){
                writer.println(setdata.get(i));
            }
            
            writer.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }//GEN-LAST:event_mnuFileSaveActionPerformed

    private void mnuFileOpenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuFileOpenActionPerformed
        JFileChooser fbox = new JFileChooser();
        JOptionPane msgbox = new JOptionPane();
        
        fbox.setFileSelectionMode(fbox.DIRECTORIES_ONLY);
        if(fbox.showOpenDialog(this) == fbox.APPROVE_OPTION){
            skinfolder=fbox.getSelectedFile();
            
            File setfile = new File(skinfolder.getPath()+skinfolder.separator+"settings.ini");
            if(setfile.exists()){
                //Read in file
                try{
                    Scanner inireader = new Scanner(setfile);
                    
                    while(inireader.hasNextLine())
                        setdata.add(inireader.nextLine());
                    
                    inireader.close();
                    loaded=true;
                    lstSkinItems.setEnabled(true);
                }
                catch(Exception e){
                    msgbox.showMessageDialog(this,"An error occured while reading the skin settings.ini file!"); //Spit out an error!
                    e.printStackTrace();
                }
            }
            else{
                msgbox.showMessageDialog(this,"The settings.ini file for this skin was not found! Did you select the skin folder?");
            }
        }
    }//GEN-LAST:event_mnuFileOpenActionPerformed

    private void mnuFileExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuFileExitActionPerformed
        this.dispose();
    }//GEN-LAST:event_mnuFileExitActionPerformed

    private void lstSkinItemsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_lstSkinItemsValueChanged
        Image tempImage;
        tabEditors.removeAll();
        switch(lstSkinItems.getSelectedIndex()){
            case 0:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"DefaultBevel.png").getImage();
                populateSliceItem("inibmpDefaultBevel","iniDefaultBevelFrame",tempImage);
                break;
            case 1:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"DefaultPressed.png").getImage();
                populateSliceItem("inibmpDefaultPressed","iniDefaultPressedFrame",tempImage);
                break;
            case 2:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Button.png").getImage();
                populateSliceItem("inibmpButton-Flat","iniButton-Flat",tempImage);
                break;
            case 3:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Button.png").getImage();
                populateSliceItem("inibmpButton-Over","iniButton-Over",tempImage);
                break;
            case 4:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Button.png").getImage();
                populateSliceItem("inibmpButton-Push","iniButton-Push",tempImage);
                break;
            case 5:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Button.png").getImage();
                populateSliceItem("inibmpButton-Disabled","iniButton-Disabled",tempImage);
                break;
            case 6:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"DisplayBox.png").getImage();
                populateSliceItem("inibmpDisplayBox","iniDisplayBoxFrame",tempImage);
                break;
            case 7:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"TextBox.png").getImage();
                populateSliceItem("inibmpTextBox","iniTextBoxFrame",tempImage);
                break;
            case 8:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Frame.png").getImage();
                populateSliceItem("inibmpFrame","iniFrame",tempImage);
                break;
            case 9:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"ItemList.png").getImage();
                populateSliceItem("inibmpItemList","iniItemListFrame",tempImage);
                break;
            case 10:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"List.png").getImage();
                populateSliceItem("inibmpList","iniListFrame",tempImage);
                break;
            case 11:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"MenuBar.png").getImage();
                populateSliceItem("inibmpMenuBar","iniMenuBarFrame",tempImage);
                break;
            case 12:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Menu.png").getImage();
                populateSliceItem("inibmpMenu","iniMenuFrame",tempImage);
                break;
            case 13:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"MenuBarButton.png").getImage();
                populateSliceItem("inibmpMenuBarButton-Flat","iniMenuBarButton-Flat",tempImage);
                break;
            case 14:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"MenuBarButton.png").getImage();
                populateSliceItem("inibmpMenuBarButton-Over","iniMenuBarButton-Over",tempImage);
                break;
            case 15:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"MenuBarButton.png").getImage();
                populateSliceItem("inibmpMenuBarButton-Push","iniMenuBarButton-Push",tempImage);
                break;
            case 16:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"Window.png").getImage();
                populateSliceItem("inibmpWindow","iniWindowFrame",tempImage);
                break;
            case 17:
                tempImage = new ImageIcon(skinfolder.getPath()+skinfolder.separator+"images"
                        +skinfolder.separator+"WindowInterior.png").getImage();
                populateSliceItem("inibmpWindowInterior","iniWindowInteriorFrame",tempImage);
                break;
            default:
                break; //Do nothing!
        }
    }//GEN-LAST:event_lstSkinItemsValueChanged
       
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JInternalFrame infPreview;
    private javax.swing.JList lstSkinItems;
    private javax.swing.JMenuBar mnuBar;
    private javax.swing.JMenu mnuFile;
    private javax.swing.JMenuItem mnuFileAbout;
    private javax.swing.JMenuItem mnuFileExit;
    private javax.swing.JMenuItem mnuFileOpen;
    private javax.swing.JMenuItem mnuFileSave;
    private javax.swing.JSeparator mnuFileSep2;
    private javax.swing.JSeparator mnuFileSeparator;
    private javax.swing.JPanel pnlPreviewHolder;
    private javax.swing.JPanel pnlWinRight;
    private javax.swing.JScrollPane scrSkinItems;
    private javax.swing.JSplitPane sptWindowPanes;
    private javax.swing.JTabbedPane tabEditors;
    // End of variables declaration//GEN-END:variables
    
}
