/*
 * ButtonPreview.java
 *
 * Created on July 14, 2006, 3:36 PM
 *
 * @author Paul Kratt
 * @desc A custom JPanel which draws a "sliceable" Trillian skin item.
 *
 * Note: This code was released as open source on January 6th 2007.
 * There is no license behind it. The only restrictions I ask are that it may
 * not be sold and that some form of credit towards me remains if you use it.
 */

package trillianskinviewer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

public class ItemPreview extends JPanel{
    SliceableItem myitm;
    
    /** Creates a new instance of ButtonPreview */
    public ItemPreview(SliceableItem itm) {
        myitm=itm;
    }

    public void paint(Graphics g) {
        super.paint(g); //Do all of the things that paint() was supposed to do.
        
        int width=this.getWidth(),height=this.getHeight();
        int tempwidth=myitm.getSliceleft()-myitm.getLeft();
        int tempheight=myitm.getSlicetop()-myitm.getTop();
        int tempx,tempy;
        
        //Fix width and height, to impose the minimum size defined by the size. This allows us to  
        //accurately simulate the "pushing over" effect seen when you make a control too small.
        if(width<myitm.getRight()-myitm.getLeft())
            width=myitm.getRight()-myitm.getLeft();
        if(height<myitm.getBottom()-myitm.getTop())
            height=myitm.getBottom()-myitm.getTop();
        
        //Draw top left corner
        g.drawImage(myitm.getItemImage(),0,0,tempwidth,tempheight,myitm.getLeft(),myitm.getTop(),myitm.getSliceleft(),myitm.getSlicetop(),this);
        
        //Draw top middle pieces
        tempwidth=myitm.getSliceright()-myitm.getSliceleft();
        tempx = myitm.getSliceleft()-myitm.getLeft();
        while(tempx<width){
            g.drawImage(myitm.getItemImage(),tempx,0,tempx+tempwidth,tempheight,myitm.getSliceleft(),myitm.getTop(),myitm.getSliceright(),myitm.getSlicetop(),this);
            tempx+=tempwidth;
        }
        
        //Draw top right corner
        tempwidth=myitm.getRight()-myitm.getSliceright();
        g.drawImage(myitm.getItemImage(),width-tempwidth,0,width,tempheight,myitm.getSliceright(),myitm.getTop(),myitm.getRight(),myitm.getSlicetop(),this);
        
        //Draw the left side
        tempwidth=myitm.getSliceleft()-myitm.getLeft();
        tempheight=myitm.getSlicebottom()-myitm.getSlicetop();
        tempy = myitm.getSlicetop()-myitm.getTop();
        while(tempy<height){
            g.drawImage(myitm.getItemImage(),0,tempy,tempwidth,tempy+tempheight,myitm.getLeft(),myitm.getSlicetop(),myitm.getSliceleft(),myitm.getSlicebottom(),this);
            tempy+=tempheight;
        }
        
        //Draw the bottom left corner
        tempheight=myitm.getBottom()-myitm.getSlicebottom();
        g.drawImage(myitm.getItemImage(),0,height-tempheight,tempwidth,height,myitm.getLeft(),myitm.getSlicetop(),myitm.getSliceleft(),myitm.getBottom(),this);
        
        //Fill in the middle now... Hmmm...
        tempwidth = myitm.getSliceright()-myitm.getSliceleft();
        tempheight = myitm.getSlicebottom()-myitm.getSlicetop();
        if(myitm.getBackblue() > 0 && myitm.getBackgreen() > 0 && myitm.getBackred() > 0){
            g.setColor(new Color(myitm.getBackred(),myitm.getBackgreen(),myitm.getBackblue()));
            g.fillRect(myitm.getSliceleft()-myitm.getLeft(),myitm.getSlicetop()-myitm.getTop(),myitm.getSliceright()-myitm.getSliceleft(),myitm.getSlicebottom()-myitm.getSlicetop());
        }
        else{
            //Fill with a pattern! (I hope this is what I'm supposed to do!)
            tempy = myitm.getSlicetop()-myitm.getTop();
            
            while(tempy<height){
                tempx = myitm.getSliceleft()-myitm.getLeft();
                while(tempx<width){
                    g.drawImage(myitm.getItemImage(),tempx,tempy,tempx+tempwidth,tempy+tempheight,myitm.getSliceleft(),myitm.getSlicetop(),myitm.getSliceright(),myitm.getSlicebottom(),this);
                    tempx+=tempwidth;
                }
                tempy+=tempheight;
            }
        }
        
        //Draw bottom middle pieces
        tempx = myitm.getSliceleft()-myitm.getLeft();
        tempheight = myitm.getBottom()-myitm.getSlicebottom();
        while(tempx<width){
            g.drawImage(myitm.getItemImage(),tempx,height-tempheight,tempx+tempwidth,height,myitm.getSliceleft(),myitm.getSlicebottom(),myitm.getSliceright(),myitm.getBottom(),this);
            tempx+=tempwidth;
        }
        
        //Draw the right side
        tempwidth=myitm.getRight()-myitm.getSliceright();
        tempheight=myitm.getSlicebottom()-myitm.getSlicetop();
        tempy = myitm.getSlicetop()-myitm.getTop();
        while(tempy<height){
            g.drawImage(myitm.getItemImage(),width-tempwidth,tempy,width,tempy+tempheight,myitm.getSliceright(),myitm.getSlicetop(),myitm.getRight(),myitm.getSlicebottom(),this);
            tempy+=tempheight;
        }
        
        //Draw the bottom right corner
        tempwidth = myitm.getRight()-myitm.getSliceright();
        tempheight=myitm.getBottom()-myitm.getSlicebottom();
        g.drawImage(myitm.getItemImage(),width-tempwidth,height-tempheight,width,height,myitm.getSliceright(),myitm.getSlicebottom(),myitm.getRight(),myitm.getBottom(),this);
        
        //Help the SliceableItem class out
        if(!myitm.knowsSize()){
            myitm.setWidth(myitm.getItemImage().getWidth(this));
            myitm.setHeight(myitm.getItemImage().getHeight(this));
        }
    }
    
}
