/*
 * SliceableItem.java
 *
 * Created on July 15, 2006, 2:14 PM
 *
 * @author Paul Kratt
 * @desc A class to hold the properties of a sliceable item.
 *
 * Note: This code was released as open source on January 6th 2007.
 * There is no license behind it. The only restrictions I ask are that it may
 * not be sold and that some form of credit towards me remains if you use it.
 */

package trillianskinviewer;

import java.awt.Color;
import java.awt.Image;
import java.util.ArrayList;
import javax.swing.ImageIcon;

public class SliceableItem {
    //Way too much information. :-(
    //I could use an array to store it, but proper variable names will make
    //the code easier to read.
    private int top,bottom,left,right; //Used image part location
    private int width,height; //If this value doesn't get populated, I'm screwed!
    private int slicetop,slicebottom,sliceleft,sliceright; //Image slice
    private int forered,foregreen,foreblue; //Foreground color
    private int backred,backgreen,backblue; //Background color
    private int line_top,line_bottom,line_left,line_right;
    private int line_slicetop,line_slicebottom,line_sliceleft,line_sliceright;
    private int line_widthtop,line_widthbottom,line_widthleft,line_widthright;
    private int line_forered,line_foregreen,line_foreblue;
    private int line_backred,line_backgreen,line_backblue;
    private Image itemImage;
    private ArrayList<String> settingData; //Settings file data, so we can write changes from here.
    private ItemPreview myviewer;
    private boolean knowsSize; //Do I know the actual size of the image?
    
    /** Creates a new instance of SliceableItem */
    public SliceableItem() {
        knowsSize=false;
    }
    
    /** Update the data in the local copy of the ini file with the current values.
     *Changes won't be written back until the user saves.
     */ 
    public void updateSaveData(){
        updateLine(line_top,top);
        updateLine(line_bottom,bottom);
        updateLine(line_left,left);
        updateLine(line_right,right);
        updateLine(line_slicetop,slicetop);
        updateLine(line_slicebottom,slicebottom);
        updateLine(line_sliceleft,sliceleft);
        updateLine(line_sliceright,sliceright);
        
        updateLine(line_widthtop,slicetop);
        updateLine(line_widthleft,sliceleft);
        updateLine(line_widthbottom,bottom-slicebottom);
        updateLine(line_widthright,right-sliceright);
        
        if(line_forered>=0 && line_foregreen>=0 && line_foreblue>=0){
            updateLine(line_forered,forered);
            updateLine(line_foregreen,foregreen);
            updateLine(line_foreblue,foreblue);
        }
        if(line_backred>=0 && line_backgreen>=0 && line_backblue>=0){
            updateLine(line_backred,backred);
            updateLine(line_backgreen,backgreen);
            updateLine(line_backblue,backblue);
        }
    }
    
    /** Update one line's value. Used by the function above this one!
     * WARNING: This function assumes that the line you give it is valid!
     **/
    private void updateLine(int line, int value){
        String templine = settingData.get(line);
        
        templine = templine.substring(0,templine.indexOf("'")+1) + Integer.toString(value) + 
                templine.substring(templine.lastIndexOf("'"),templine.length());
        settingData.set(line,templine);
    }
    
    /** Refresh the viewer. Call this after you've changed values and want to see the results. **/
    public void refreshViewer(){
        myviewer.repaint();
    }
    
    /** If some of the slice bounds have stupid values,
     * such as the right being to the left of the left, then this resets them
     * to values that will at least allow the skin to draw properly.
     */
    public void boundsCheck(){
        if(top<0)
            top=0;
        if(left<0)
            left=0;
        if(bottom<3)
            bottom=3;
        if(right<3)
            right=3;
        if(knowsSize){
            if(right>width)
                right=width;
            if(bottom>height)
                bottom=height;
        }
        if(slicetop<=top)
            slicetop=top+1;
        if(slicebottom>=bottom)
            slicebottom=bottom-1;
        if(sliceleft<=left)
            sliceleft=left+1;
        if(sliceright>=right)
            sliceright=right-1;
        if(slicetop>=slicebottom)
            slicebottom=slicetop+1;
        if(sliceleft>=sliceright)
            sliceright=sliceleft+1;
    }

    //Below are a bunch of uncommented setter's and getters.
    
    public int getTop() {
        return top;
    }

    public void setTop(int top) {
        this.top = top;
    }

    public int getBottom() {
        return bottom;
    }

    public void setBottom(int bottom) {
        this.bottom = bottom;
    }

    public int getLeft() {
        return left;
    }

    public void setLeft(int left) {
        this.left = left;
    }

    public int getRight() {
        return right;
    }

    public void setRight(int right) {
        this.right = right;
    }

    public int getSlicetop() {
        return slicetop;
    }

    public void setSlicetop(int slicetop) {
        this.slicetop = slicetop;
    }

    public int getSlicebottom() {
        return slicebottom;
    }

    public void setSlicebottom(int slicebottom) {
        this.slicebottom = slicebottom;
    }

    public int getSliceleft() {
        return sliceleft;
    }

    public void setSliceleft(int sliceleft) {
        this.sliceleft = sliceleft;
    }

    public int getSliceright() {
        return sliceright;
    }

    public void setSliceright(int sliceright) {
        this.sliceright = sliceright;
    }

    public int getForered() {
        return forered;
    }

    public void setForered(int forered) {
        this.forered = forered;
    }

    public int getForegreen() {
        return foregreen;
    }

    public void setForegreen(int foregreen) {
        this.foregreen = foregreen;
    }

    public int getForeblue() {
        return foreblue;
    }

    public void setForeblue(int foreblue) {
        this.foreblue = foreblue;
    }

    public int getBackred() {
        return backred;
    }

    public void setBackred(int backred) {
        this.backred = backred;
    }

    public int getBackgreen() {
        return backgreen;
    }

    public void setBackgreen(int backgreen) {
        this.backgreen = backgreen;
    }

    public int getBackblue() {
        return backblue;
    }

    public void setBackblue(int backblue) {
        this.backblue = backblue;
    }

    public int getLine_top() {
        return line_top;
    }

    public void setLine_top(int line_top) {
        this.line_top = line_top;
    }

    public int getLine_bottom() {
        return line_bottom;
    }

    public void setLine_bottom(int line_bottom) {
        this.line_bottom = line_bottom;
    }

    public int getLine_left() {
        return line_left;
    }

    public void setLine_left(int line_left) {
        this.line_left = line_left;
    }

    public int getLine_right() {
        return line_right;
    }

    public void setLine_right(int line_right) {
        this.line_right = line_right;
    }

    public int getLine_slicetop() {
        return line_slicetop;
    }

    public void setLine_slicetop(int line_slicetop) {
        this.line_slicetop = line_slicetop;
    }

    public int getLine_slicebottom() {
        return line_slicebottom;
    }

    public void setLine_slicebottom(int line_slicebottom) {
        this.line_slicebottom = line_slicebottom;
    }

    public int getLine_sliceleft() {
        return line_sliceleft;
    }

    public void setLine_sliceleft(int line_sliceleft) {
        this.line_sliceleft = line_sliceleft;
    }

    public int getLine_sliceright() {
        return line_sliceright;
    }

    public void setLine_sliceright(int line_sliceright) {
        this.line_sliceright = line_sliceright;
    }

    public int getLine_widthtop() {
        return line_widthtop;
    }

    public void setLine_widthtop(int line_widthtop) {
        this.line_widthtop = line_widthtop;
    }

    public int getLine_widthbottom() {
        return line_widthbottom;
    }

    public void setLine_widthbottom(int line_widthbottom) {
        this.line_widthbottom = line_widthbottom;
    }

    public int getLine_widthleft() {
        return line_widthleft;
    }

    public void setLine_widthleft(int line_widthleft) {
        this.line_widthleft = line_widthleft;
    }

    public int getLine_widthright() {
        return line_widthright;
    }

    public void setLine_widthright(int line_widthright) {
        this.line_widthright = line_widthright;
    }

    public int getLine_forered() {
        return line_forered;
    }

    public void setLine_forered(int line_forered) {
        this.line_forered = line_forered;
    }

    public int getLine_foregreen() {
        return line_foregreen;
    }

    public void setLine_foregreen(int line_foregreen) {
        this.line_foregreen = line_foregreen;
    }

    public int getLine_foreblue() {
        return line_foreblue;
    }

    public void setLine_foreblue(int line_foreblue) {
        this.line_foreblue = line_foreblue;
    }

    public int getLine_backred() {
        return line_backred;
    }

    public void setLine_backred(int line_backred) {
        this.line_backred = line_backred;
    }

    public int getLine_backgreen() {
        return line_backgreen;
    }

    public void setLine_backgreen(int line_backgreen) {
        this.line_backgreen = line_backgreen;
    }

    public int getLine_backblue() {
        return line_backblue;
    }

    public void setLine_backblue(int line_backblue) {
        this.line_backblue = line_backblue;
    }

    public Image getItemImage() {
        return itemImage;
    }

    public void setItemImage(Image itemImage) {
        this.itemImage = itemImage;
    }

    public ArrayList<String> getSettingData() {
        return settingData;
    }

    public void setSettingData(ArrayList<String> settingData) {
        this.settingData = settingData;
    }

    public void setWidth(int width) {
        this.width = width;
        if(width >0 && height > 0)
            knowsSize=true;
    }

    public void setHeight(int height) {
        this.height = height;
        if(width >0 && height > 0)
            knowsSize=true;
    }

    public void setMyviewer(ItemPreview myviewer) {
        this.myviewer = myviewer;
    }

    public boolean knowsSize() {
        return knowsSize;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }
    
}
